@@ -5,6 +5,7 @@
 package regtest
 
 import (
+	"encoding/json"
 	"io"
 	"path"
 	"testing"
@@ -229,6 +230,15 @@ func (e *Env) DocumentLink(name string) []protocol.DocumentLink {
 	return links
 }
 
+func (e *Env) DocumentHighlight(name string, pos fake.Pos) []protocol.DocumentHighlight {
+	e.T.Helper()
+	highlights, err := e.Editor.DocumentHighlight(e.Ctx, name, pos)
+	if err != nil {
+		e.T.Fatal(err)
+	}
+	return highlights
+}
+
 func checkIsFatal(t *testing.T, err error) {
 	t.Helper()
 	if err != nil && !errors.Is(err, io.EOF) && !errors.Is(err, io.ErrClosedPipe) {
@@ -312,6 +322,7 @@ func (e *Env) CodeLens(path string) []protocol.CodeLens {
 // ExecuteCodeLensCommand executes the command for the code lens matching the
 // given command name.
 func (e *Env) ExecuteCodeLensCommand(path string, cmd command.Command) {
+	e.T.Helper()
 	lenses := e.CodeLens(path)
 	var lens protocol.CodeLens
 	var found bool
@@ -324,10 +335,32 @@ func (e *Env) ExecuteCodeLensCommand(path string, cmd command.Command) {
 	if !found {
 		e.T.Fatalf("found no command with the ID %s", cmd.ID())
 	}
-	if _, err := e.Editor.ExecuteCommand(e.Ctx, &protocol.ExecuteCommandParams{
+	e.ExecuteCommand(&protocol.ExecuteCommandParams{
 		Command:   lens.Command.Command,
 		Arguments: lens.Command.Arguments,
-	}); err != nil {
+	}, nil)
+}
+
+func (e *Env) ExecuteCommand(params *protocol.ExecuteCommandParams, result interface{}) {
+	e.T.Helper()
+	response, err := e.Editor.ExecuteCommand(e.Ctx, params)
+	if err != nil {
+		e.T.Fatal(err)
+	}
+	if result == nil {
+		return
+	}
+	// Hack: The result of an executeCommand request will be unmarshaled into
+	// maps. Re-marshal and unmarshal into the type we expect.
+	//
+	// This could be improved by generating a jsonrpc2 command client from the
+	// command.Interface, but that should only be done if we're consolidating
+	// this part of the tsprotocol generation.
+	data, err := json.Marshal(response)
+	if err != nil {
+		e.T.Fatal(err)
+	}
+	if err := json.Unmarshal(data, result); err != nil {
 		e.T.Fatal(err)
 	}
 }
@@ -364,9 +397,9 @@ func (e *Env) AcceptCompletion(path string, pos fake.Pos, item protocol.Completi
 
 // CodeAction calls testDocument/codeAction for the given path, and calls
 // t.Fatal if there are errors.
-func (e *Env) CodeAction(path string) []protocol.CodeAction {
+func (e *Env) CodeAction(path string, diagnostics []protocol.Diagnostic) []protocol.CodeAction {
 	e.T.Helper()
-	actions, err := e.Editor.CodeAction(e.Ctx, path, nil)
+	actions, err := e.Editor.CodeAction(e.Ctx, path, nil, diagnostics)
 	if err != nil {
 		e.T.Fatal(err)
 	}
