@@ -23,15 +23,13 @@ import * as u from './util';
 import { constName, getComments, goName, loc, strKind } from './util';
 
 var program: ts.Program;
-// eslint-disable-next-line no-unused-vars
-var checker: ts.TypeChecker;
 
 function parse() {
   // this won't complain if some fnames don't exist
   program = ts.createProgram(
     u.fnames,
     { target: ts.ScriptTarget.ES2018, module: ts.ModuleKind.CommonJS });
-  checker = program.getTypeChecker();  // finish type checking and assignment
+  program.getTypeChecker();  // finish type checking and assignment
 }
 
 // ----- collecting information for RPCs
@@ -56,7 +54,7 @@ function findRPCs(node: ts.Node) {
 
   if (!ts.isModuleBlock(node.body)) {
     throw new Error(
-      `expected ModuleBody got ${strKind(node.body)} at ${loc(node)}`);
+      `expected ModuleBlock got ${strKind(node.body)} at ${loc(node)}`);
   }
   let x: ts.ModuleBlock = node.body;
   // The story is to expect const method = 'textDocument/implementation'
@@ -103,7 +101,7 @@ function findRPCs(node: ts.Node) {
           ptypes.set(rpc, [nn.typeArguments[0], nn.typeArguments[1]]);
           break;
         default:
-          throw new Error(`${nn.typeArguments.length} at ${loc(nn)}`);
+          throw new Error(`${nn.typeArguments?.length} at ${loc(nn)}`);
       }
     }
   }
@@ -426,7 +424,7 @@ function checkOnce() {
 
 // helper function to find underlying types
 // eslint-disable-next-line no-unused-vars
-function underlying(n: ts.Node, f: (n: ts.Node) => void) {
+function underlying(n: ts.Node | undefined, f: (n: ts.Node) => void) {
   if (!n) return;
   const ff = function (n: ts.Node) {
     underlying(n, f);
@@ -543,7 +541,9 @@ function sameType(a: ts.TypeNode, b: ts.TypeNode): boolean {
   }
   throw new Error(`546 sameType? ${strKind(a)} ${strKind(b)}`);
 }
-
+type CreateMutable<Type> = {
+  -readonly [Property in keyof Type]: Type[Property];
+};
 type propMap = Map<string, ts.PropertySignature>;
 function propMapSet(pm: propMap, name: string, v: ts.PropertySignature) {
   if (!pm.get(name)) {
@@ -557,16 +557,16 @@ function propMapSet(pm: propMap, name: string, v: ts.PropertySignature) {
     return;
   }
   if (ts.isTypeReferenceNode(a) && ts.isTypeLiteralNode(b)) {
-    const x = mergeTypeRefLit(name, a, b);
-    const fake: Object = v;
+    const x = mergeTypeRefLit(a, b);
+    const fake: CreateMutable<ts.PropertySignature> = v;
     fake['type'] = x;
     check(fake as ts.PropertySignature, '565');
     pm.set(name, fake as ts.PropertySignature);
     return;
   }
   if (ts.isTypeLiteralNode(a) && ts.isTypeLiteralNode(b)) {
-    const x = mergeTypeLitLit(name, a, b);
-    const fake: Object = v;
+    const x = mergeTypeLitLit(a, b);
+    const fake: CreateMutable<ts.PropertySignature> = v;
     fake['type'] = x;
     check(fake as ts.PropertySignature, '578');
     pm.set(name, fake as ts.PropertySignature);
@@ -575,7 +575,8 @@ function propMapSet(pm: propMap, name: string, v: ts.PropertySignature) {
   console.log(`577 ${pm.get(name).getText()}\n${v.getText()}`);
   throw new Error(`578 should merge ${strKind(a)} and ${strKind(b)} for ${name}`);
 }
-function addToProperties(pm: propMap, tn: ts.TypeNode, prefix = '') {
+function addToProperties(pm: propMap, tn: ts.TypeNode | undefined, prefix = '') {
+  if (!tn) return;
   if (ts.isTypeReferenceNode(tn)) {
     const d = seenTypes.get(goName(tn.typeName.getText()));
     if (tn.typeName.getText() === 'T') return;
@@ -602,9 +603,9 @@ function addToProperties(pm: propMap, tn: ts.TypeNode, prefix = '') {
     });
   }
 }
-function deepProperties(d: Data): propMap {
+function deepProperties(d: Data): propMap | undefined {
   let properties: propMap = new Map<string, ts.PropertySignature>();
-  if (!d.alias || !ts.isIntersectionTypeNode(d.alias)) return;
+  if (!d.alias || !ts.isIntersectionTypeNode(d.alias)) return undefined;
   d.alias.types.forEach((ts) => addToProperties(properties, ts));
   return properties;
 }
@@ -625,19 +626,19 @@ function mergeAlias(d: Data) {
   d.properties = ts.factory.createNodeArray(v);
 }
 
-function mergeTypeLitLit(name: string, a: ts.TypeLiteralNode, b: ts.TypeLiteralNode): ts.TypeLiteralNode {
+function mergeTypeLitLit(a: ts.TypeLiteralNode, b: ts.TypeLiteralNode): ts.TypeLiteralNode {
   const v = new Map<string, ts.TypeElement>(); // avoid duplicates
   a.members.forEach((te) => v.set(te.name.getText(), te));
   b.members.forEach((te) => v.set(te.name.getText(), te));
   const x: ts.TypeElement[] = [];
   v.forEach((te) => x.push(te));
-  const fake: Object = a;
-  fake['members'] = x;
+  const fake: CreateMutable<ts.TypeLiteralNode> = a;
+  fake['members'] = ts.factory.createNodeArray(x);
   check(fake as ts.TypeLiteralNode, '643');
   return fake as ts.TypeLiteralNode;
 }
 
-function mergeTypeRefLit(name: string, a: ts.TypeReferenceNode, b: ts.TypeLiteralNode): ts.TypeLiteralNode {
+function mergeTypeRefLit(a: ts.TypeReferenceNode, b: ts.TypeLiteralNode): ts.TypeLiteralNode {
   const d = seenTypes.get(goName(a.typeName.getText()));
   if (!d) throw new Error(`644 name ${a.typeName.getText()} not found`);
   const typ = d.me;
@@ -649,10 +650,10 @@ function mergeTypeRefLit(name: string, a: ts.TypeReferenceNode, b: ts.TypeLitera
   v.forEach((te) => x.push(te));
 
   const w = ts.factory.createNodeArray(x);
-  const fk: Object = b;
+  const fk: CreateMutable<ts.TypeLiteralNode> = b;
   fk['members'] = w;
-  fk['members']['pos'] = b.members.pos;
-  fk['members']['end'] = b.members.end;
+  (fk['members'] as { pos: number })['pos'] = b.members.pos;
+  (fk['members'] as { end: number })['end'] = b.members.end;
   check(fk as ts.TypeLiteralNode, '662');
   return fk as ts.TypeLiteralNode;
 }
@@ -801,7 +802,8 @@ function goTypeAlias(d: Data, nm: string) {
 }
 
 // return a go type and maybe an assocated javascript tag
-function goType(n: ts.TypeNode, nm: string): string {
+function goType(n: ts.TypeNode | undefined, nm: string): string {
+  if (!n) throw new Error(`goType undefined for ${nm}`);
   if (n.getText() == 'T') return 'interface{}';  // should check it's generic
   if (ts.isTypeReferenceNode(n)) {
     switch (n.getText()) {
@@ -1305,8 +1307,8 @@ function output(side: side) {
     side.fd = fs.openSync(side.outputFile, 'w');
   }
   const f = function (s: string) {
-    fs.writeSync(side.fd, s);
-    fs.writeSync(side.fd, '\n');
+    fs.writeSync(side.fd!, s);
+    fs.writeSync(side.fd!, '\n');
   };
   f(u.computeHeader(false));
   f(`
