@@ -215,6 +215,10 @@ type View interface {
 	// Folder returns the folder with which this view was created.
 	Folder() span.URI
 
+	// TempWorkspace returns the folder this view uses for its temporary
+	// workspace module.
+	TempWorkspace() span.URI
+
 	// Shutdown closes this view, and detaches it from its session.
 	Shutdown(ctx context.Context)
 
@@ -291,8 +295,13 @@ type TidiedModule struct {
 // of the client.
 // A session may have many active views at any given time.
 type Session interface {
-	// NewView creates a new View, returning it and its first snapshot.
-	NewView(ctx context.Context, name string, folder, tempWorkspaceDir span.URI, options *Options) (View, Snapshot, func(), error)
+	// ID returns the unique identifier for this session on this server.
+	ID() string
+	// NewView creates a new View, returning it and its first snapshot. If a
+	// non-empty tempWorkspace directory is provided, the View will record a copy
+	// of its gopls workspace module in that directory, so that client tooling
+	// can execute in the same main module.
+	NewView(ctx context.Context, name string, folder, tempWorkspace span.URI, options *Options) (View, Snapshot, func(), error)
 
 	// Cache returns the cache that created this session, for debugging only.
 	Cache() interface{}
@@ -522,7 +531,7 @@ type Analyzer struct {
 
 	// ActionKind is the kind of code action this analyzer produces. If
 	// unspecified the type defaults to quickfix.
-	ActionKind protocol.CodeActionKind
+	ActionKind []protocol.CodeActionKind
 }
 
 func (a Analyzer) IsEnabled(view View) bool {
